import requests
import os
import json
import logging
from kis_auth import refresh_token, load_token

def get_search_conditions():
    access_token = load_token()
    api_key = os.getenv("MY_APP")
    api_secret = os.getenv("MY_SEC")
    user_id = os.getenv("USER_ID")

    url = "https://openapi.koreainvestment.com:9443/uapi/domestic-stock/v1/quotations/psearch-title"
    headers = {
        'content-type': 'application/json; charset=utf-8',
        'authorization': f'Bearer {access_token}',
        'appkey': api_key,
        'appsecret': api_secret,
        'tr_id': 'HHKST03900300'
    }
    params = {'user_id': user_id}

    response = requests.get(url, headers=headers, params=params)
    
    if response.status_code == 200:
        return response.json().get('output2', [])
    elif response.status_code == 401:  # Unauthorized
        logging.info("Token expired during get_search_conditions request. Requesting new token.")
        access_token = refresh_token()
        return get_search_conditions()  # Retry with new token
    else:
        logging.error(f"Error fetching search conditions: {response.text}")
        return []

def fetch_stocks_by_condition(condition_seq):
    access_token = load_token()
    api_key = os.getenv("MY_APP")
    api_secret = os.getenv("MY_SEC")
    user_id = os.getenv("USER_ID")

    url = f"https://openapi.koreainvestment.com:9443/uapi/domestic-stock/v1/quotations/psearch-result?user_id={user_id}&seq={condition_seq}"
    headers = {
        'content-type': 'application/json; charset=utf-8',
        'authorization': f'Bearer {access_token}',
        'appkey': api_key,
        'appsecret': api_secret,
        'tr_id': 'HHKST03900400'
    }
    
    response = requests.get(url, headers=headers)
    
    if response.status_code == 200:
        return [stock['code'] for stock in response.json().get('output2', [])]
    elif response.status_code == 401:  # Unauthorized
        logging.info("Token expired during fetch_stocks_by_condition request. Requesting new token.")
        access_token = refresh_token()
        return fetch_stocks_by_condition(condition_seq)  # Retry with new token
    else:
        logging.error(f"Error fetching stocks by condition {condition_seq}: {response.text}")
        return []

def fetch_stock_data(stock_code):
    access_token = load_token()
    api_key = os.getenv("MY_APP")
    api_secret = os.getenv("MY_SEC")
    
    url = f"https://openapi.koreainvestment.com:9443/uapi/domestic-stock/v1/quotations/inquire-price?FID_COND_MRKT_DIV_CODE=J&FID_INPUT_ISCD={stock_code}"
    headers = {
        'content-type': 'application/json; charset=utf-8',
        'authorization': f'Bearer {access_token}',
        'appkey': api_key,
        'appsecret': api_secret,
        'tr_id': 'FHKST01010100'
    }

    response = requests.get(url, headers=headers)
    
    if response.status_code == 200:
        return response.json()
    elif response.status_code == 401:  # Unauthorized
        logging.info("Token expired during fetch_stock_data request. Requesting new token.")
        access_token = refresh_token()
        return fetch_stock_data(stock_code)  # Retry with new token
    else:
        logging.error(f"Error fetching stock data for {stock_code}: {response.text}")
        return {}

# Ensure that the initial token is loaded
load_token()
