
import backtrader as bt

class OptimizedStrategy(bt.Strategy):
    params = (
        ('ma_period', 15),
        ('rsi_period', 14),
        ('rsi_upper', 70),
        ('rsi_lower', 30),
    )

    def __init__(self):
        self.ma = bt.indicators.SimpleMovingAverage(self.data.close, period=self.params.ma_period)
        self.rsi = bt.indicators.RSI_SMA(self.data.close, period=self.params.rsi_period)

    def next(self):
        if self.rsi < self.params.rsi_lower and self.data.close > self.ma:
            self.buy(size=100)
        elif self.rsi > self.params.rsi_upper and self.data.close < self.ma:
            self.sell(size=100)

def optimize_strategy(data):
    cerebro = bt.Cerebro(optreturn=False)
    cerebro.optstrategy(OptimizedStrategy, ma_period=range(10, 50), rsi_period=range(10, 30))
    cerebro.adddata(data)
    cerebro.run(maxcpus=1)
