import logging
from kis_auth import refresh_token, load_token
import os
import requests
from dotenv import load_dotenv

# Load environment variables from the specified path
dotenv_path = r'C:\Users\user\KIS\config\.env'
load_dotenv(dotenv_path)

def fetch_single_stock_data(stock_code):
    api_key = os.getenv("MY_APP")
    api_secret = os.getenv("MY_SEC")
    user_id = os.getenv("USER_ID")
    access_token = load_token()

    if not access_token:
        logging.error("Access token is missing. Refreshing token.")
        access_token = refresh_token()

    url = f"https://openapi.koreainvestment.com:9443/uapi/domestic-stock/v1/quotations/inquire-price?user_id={user_id}&fid_cond_mrkt_div_code=J&fid_input_iscd={stock_code}"
    headers = {
        'content-type': 'application/json; charset=utf-8',
        'authorization': f'Bearer {access_token}',
        'appkey': api_key,
        'appsecret': api_secret,
        'tr_id': 'FHKST01010100'
    }

    logging.info(f"Fetching data for stock {stock_code}")
    logging.info(f"Request URL: {url}")
    logging.info(f"Request Headers: {headers}")

    response = requests.get(url, headers=headers)
    logging.info(f"Response: {response.json()}")

    if response.status_code == 200:
        logging.info(f"Successfully fetched data for stock {stock_code}: {response.json()}")
        return response.json().get('output', {})
    else:
        logging.error(f"Error fetching data for stock {stock_code}: {response.text}")
        if response.json().get("msg_cd") == "EGW00121":  # If token is invalid
            logging.info("Token is invalid. Refreshing token.")
            access_token = refresh_token()
            headers['authorization'] = f'Bearer {access_token}'
            response = requests.get(url, headers=headers)
            logging.info(f"Response after token refresh: {response.json()}")
            if response.status_code == 200:
                logging.info(f"Successfully fetched data for stock {stock_code} after token refresh: {response.json()}")
                return response.json().get('output', {})
        return None

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    test_stock_code = "005930"  # Example stock code, replace with any valid stock code
    stock_data = fetch_single_stock_data(test_stock_code)
    if stock_data:
        logging.info(f"Fetched data for stock {test_stock_code}: {stock_data}")
    else:
        logging.error(f"Failed to fetch data for stock {test_stock_code}")
