import pandas as pd
import numpy as np

def calculate_ema(data, span):
    data[f'EMA_{span}'] = data['Close'].ewm(span=span, adjust=False).mean()
    return data

def calculate_rsi(data, window=14):
    delta = data['Close'].diff(1)
    gain = (delta.where(delta > 0, 0)).fillna(0)
    loss = (-delta.where(delta < 0, 0)).fillna(0)
    avg_gain = gain.rolling(window=window, min_periods=1).mean()
    avg_loss = loss.rolling(window=window, min_periods=1).mean()
    rs = avg_gain / avg_loss
    rsi = 100 - (100 / (1 + rs))
    data['RSI'] = rsi
    return data

def calculate_macd(data, short_span=12, long_span=26, signal_span=9):
    data['MACD'] = data['Close'].ewm(span=short_span, adjust=False).mean() - data['Close'].ewm(span=long_span, adjust=False).mean()
    data['Signal_Line'] = data['MACD'].ewm(span=signal_span, adjust=False).mean()
    return data

def calculate_bollinger_bands(data, window=20, num_std_dev=2):
    data['MA20'] = data['Close'].rolling(window=window).mean()
    data['BB_upper'] = data['MA20'] + (data['Close'].rolling(window=window).std() * num_std_dev)
    data['BB_lower'] = data['MA20'] - (data['Close'].rolling(window=window).std() * num_std_dev)
    return data

def moving_average_strategy(data):
    data = calculate_ema(data, 10)
    data = calculate_ema(data, 20)
    data = calculate_rsi(data)
    data = calculate_macd(data)
    data = calculate_bollinger_bands(data)
    return data
