from dotenv import load_dotenv
import os
import yaml

def check_configuration():
    load_dotenv()
    required_env_vars = ['SECRET_KEY', 'ACCESS_TOKEN']
    missing_vars = [var for var in required_env_vars if os.getenv(var) is None]

    if missing_vars:
        raise EnvironmentError(f"Missing required environment variables: {', '.join(missing_vars)}")

    with open('config.yaml', 'r', encoding='utf-8') as file:
        config = yaml.safe_load(file)
        required_keys = ['api_config', 'endpoints']
        missing_keys = [key for key in required_keys if key not in config]

        if missing_keys:
            raise KeyError(f"Missing required configuration keys: {', '.join(missing_keys)}")

    print("All configurations are correctly set up.")

check_configuration()
