import pandas as pd
from io import StringIO
import json
from collections import namedtuple

from trading_strategies import tr_plans, contract_sub_df, reserved_cols
from utils.encryption_utils import aes_cbc_base64_dec

def parse_message(data):
    global executed_df
    d1 = data.split("|")
    dp_ = None
    hcols = []

    if len(d1) >= 4:
        tr_id = d1[1]
        if tr_id == KIS_WSReq.CONTRACT:  # 실시간체결
            hcols = contract_cols
        elif tr_id == KIS_WSReq.BID_ASK: # 실시간호가
            hcols = bid_ask_cols
        elif tr_id == KIS_WSReq.NOTICE:  # 계좌체결통보
            hcols = notice_cols
        else:
            pass

        if tr_id in (KIS_WSReq.CONTRACT, KIS_WSReq.BID_ASK):
            dp_ = pd.read_csv(StringIO(d1[3]), header=None, sep='^', names=hcols, dtype=object)
            dp_['TICK_HOUR'] = _today__ + dp_['TICK_HOUR']
            dp_['TICK_HOUR'] = pd.to_datetime(dp_['TICK_HOUR'], format='%Y%m%d%H%M%S', errors='coerce')
        else:
            dp_ = pd.read_csv(StringIO(aes_cbc_base64_dec(_ekey, _iv, d1[3])), header=None, sep='^', names=hcols, dtype=object)

        if tr_id == KIS_WSReq.CONTRACT:
            stock_code = dp_[dp_.columns[0]].values.tolist()[0]
            df2_ = dp_[reserved_cols]
            selected_df = contract_sub_df.get(stock_code)
            if selected_df is not None and not selected_df.dropna().empty:
                dft_ = pd.concat([selected_df, df2_], axis=0, ignore_index=True)
            else:
                dft_ = df2_
            contract_sub_df[stock_code] = dft_

            val1 =  dp_['STCK_PRPR'].tolist()[0]
            tr_plans[stock_code].push(int(val1))
            rt_data = kb.get_inquire_psbl_order(pdno=stock_code, ord_unpr=val1)
            ord_qty = rt_data[0]['nrcvb_buy_qty']
            print("[미수없는매수주문가능수량!] : " + ord_qty)
            print("매수/매도 조건 주문 : " + val1)

        elif tr_id == KIS_WSReq.NOTICE:
            executed_df = pd.concat([executed_df, dp_], axis=0, ignore_index=True)
        else:
            pass
    else:
        print(f"Data length error...{data}")

def get_system_response(data):
    global _iv
    global _ekey

    isPingPong = False
    isUnSub = False
    isOk = False
    tr_msg = None
    tr_key = None

    rdic = json.loads(data)

    tr_id = rdic['header']['tr_id']
    if tr_id != "PINGPONG": tr_key = rdic['header']['tr_key']
    if rdic.get("body", None) is not None:
        isOk = True if rdic["body"]["rt_cd"] == "0" else False
        tr_msg = rdic["body"]["msg1"]
        if 'output' in rdic["body"]:
            _iv = rdic["body"]["output"]["iv"]
            _ekey = rdic["body"]["output"]["key"]
        isUnSub = True if tr_msg[:5] == "UNSUB" else False
    else:
        isPingPong = True if tr_id == "PINGPONG" else False

    nt2 = namedtuple('SysMsg', ['isOk', 'tr_id', 'tr_key', 'isUnSub', 'isPingPong'])
    d = {
        'isOk': isOk,
        'tr_id': tr_id,
        'tr_key': tr_key,
        'isUnSub': isUnSub,
        'isPingPong': isPingPong
    }

    return nt2(**d)
