
import os
import time
import requests
from dotenv import load_dotenv
from datetime import datetime, timedelta

load_dotenv()

class TokenManager:
    def __init__(self):
        self.token = None
        self.token_expiry = None
        self.refresh_interval = timedelta(hours=6)
        self.token_endpoint = os.getenv('TOKEN_ENDPOINT')
        self.client_id = os.getenv('CLIENT_ID')
        self.client_secret = os.getenv('CLIENT_SECRET')
        self.get_token()

    def get_token(self):
        response = requests.post(self.token_endpoint, data={
            'grant_type': 'client_credentials',
            'client_id': self.client_id,
            'client_secret': self.client_secret
        })

        if response.status_code == 200:
            token_data = response.json()
            self.token = token_data['access_token']
            self.token_expiry = datetime.now() + timedelta(seconds=token_data['expires_in'])
        else:
            raise Exception('Failed to obtain access token')

    def is_token_valid(self):
        if self.token is None or datetime.now() >= self.token_expiry:
            return False
        return True

    def ensure_token(self):
        if not self.is_token_valid() or datetime.now() >= self.token_expiry - self.refresh_interval:
            self.get_token()

    def get_headers(self):
        self.ensure_token()
        return {
            'Authorization': f'Bearer {self.token}',
            'Content-Type': 'application/json'
        }
